from ika import *
import data
import weapons
import gfx

screen = Video
plainfnt = Font( "./graphics/plain.fnt" )

TRUE = 1
FALSE = 0

BG = 0		# Won't interfere with other crafts
PLAYER = 1	# Damages enemy crafts
ENEMY = 2	# Damages player crafts
BOTH = 3	# Damages enemy and player crafts

# Status "alignments"
NORMAL = 0
SHIELD = 1
HIT = 2
BURN = 3	# Lose health a while
FREEZE = 4	# Can't control ship, moves constant
SHOCK = 5	# Can't fire

# Objects can push smaller or same size objects!
# (Objects without movescripts won't move even when pushed, though!)
VOID = 0	#unmovable, projectiles mostly
COLLECTABLE = 10
SMALL = 20
MEDIUM = 30
LARGE = 40
HUGE = 50 	#unmovable, boss


class Coords:
	def __init__( self, x, y ):
		self.x = x
		self.y = y
	
	def __tuple__( self ):
		return self.x, self.y

# Class for storing a specific craft of a defined type
class Particle:
	type = ''
	x = y = 0.0
	xSpeed = ySpeed = 0.0
	xAcc = yAcc = 0.0
	dir = 0		# for use in the movescripts
	armor = 0
	frame = 0
	shield = 0
	status = NORMAL
	genused = [0,0,0,0]
	statuscounter = 0
	side = BOTH
	armormax = 100000
	lifecounter = 0
	lifetime = 0 # infinite
	spawnondeath = None
	spawnonfreeze = None
	lasthit = None
	
	def __init__(self, type, x=0.0, y=0.0, xSpeed=0, ySpeed=0, xAcc=0, yAcc=0, moveScript=None, spawnondeath=None, spawnonfreeze=None, created = 0, lifetime=0 ):
		self.type = type
		self.x = x*1.0
		self.y = y*1.0
		self.spawnondeath = spawnondeath
		self.spawnonfreeze = spawnonfreeze
		self.ySpeed = ySpeed
		self.xSpeed = xSpeed
		self.xAcc = xAcc
		self.yAcc = yAcc
		self.armor = data.info[type].armormax
		self.lifetime = lifetime
		self.created = created
		if moveScript != None:
			self.moveScript = moveScript
		
	def tick( self, missiondata ):
		sprites = missiondata.sprites
		if self.lifetime > 0:
			self.lifecounter += 1
			if self.lifecounter == self.lifetime:
				self.removeself( missiondata )
		if self.statuscounter:
			self.statuscounter -= 1
			if self.statuscounter == 0:
				self.status = NORMAL
	
	def push(self, xforce, yforce):
		selfinfo = data.info[self.type]
		if selfinfo.size != HUGE and selfinfo.size != VOID:
			self.xSpeed += xforce*1.0/selfinfo.size
			self.ySpeed += yforce*1.0/selfinfo.size
			
	def crash(self, xforce, yforce):
		selfinfo = data.info[self.type]
		if selfinfo.size != HUGE and selfinfo.size != VOID:
			self.xSpeed = (self.xSpeed+xforce*1.0/selfinfo.size)/2
			self.ySpeed = (self.ySpeed+yforce*1.0/selfinfo.size)/2
		
	def blit(self):
		data.info[self.type].blit( self )
	
	# I can hook a movescript to here! Joy!
	def moveScript(self,a,b):
		pass

	def attackScript(self,a,b):
		pass

	def move(self):
		self.x += self.xSpeed
		self.y += self.ySpeed
		size = data.info[self.type]
		if self.x < -size.w*2 or self.x>screen.xres+size.w*2 or self.y < -size.h*2  or self.y > screen.yres+size.h*2:
			return 2
		if self.x<size.w/2 or self.x>screen.xres-size.w/2 or self.y<size.h/2 or self.y>screen.yres-size.h/2:
			return 1
		return 0
			
	def checkHits(self, missiondata, sprites ):
		selfdata = data.info[self.type]
		target = BOTH
		if selfdata.side == ENEMY:
			target = PLAYER
		if selfdata.side == PLAYER:
			target = ENEMY
		if self.type == 'explosion':
			selfdata = self
		for victim in sprites:
			victimdata = data.info[victim.type]
			if victim.type == 'explosion':
				victimdata = victim
			if ((victimdata.obstructing and selfdata.obstructing) or selfdata.projectile or victimdata.projectile) and \
			(target == BOTH or victimdata.side == target or victimdata.side == BOTH) \
			and abs(victim.x-self.x) < (selfdata.w + victimdata.w)/2 and abs(victim.y-self.y) < (selfdata.h+victimdata.h)/2:
				if victimdata.size > 0:
					self.crash( (self.x-victim.x)*victimdata.size/25.0,(self.y-victim.y)*victimdata.size/25.0 )
				if selfdata.size > 0:
					victim.crash( (victim.x-self.x)*selfdata.size/25.0,(victim.y-self.y)*selfdata.size/25.0 )
				victim.takedamage( selfdata.dmggiven, selfdata.statusgiven, missiondata )
				victim.lasthit = selfdata.side
				self.lasthit = victimdata.side
				if self.takedamage( victimdata.dmggiven, victimdata.statusgiven, missiondata ):
					return
	
	def takedamage( self, damage, status, missiondata ):
		sprites = missiondata.sprites
		if self.shield > 0:
			self.status = SHIELD
			if self.shield < damage:
				self.status = status
				self.armor -= damage - self.shield
				self.shield = 0
			else:
				self.shield -= damage
		else:
			if status == HIT and self.status == NORMAL:
				self.status = status
			elif status != HIT:
				self.status = status
			if self.status == FREEZE and not data.info[self.type].freezable:
				self.status = HIT
			if self.status == FREEZE and data.info[self.type].statusgiven == BURN:
				self.armor = 0
			self.armor -= damage
		if self.status == HIT:
			self.statuscounter = 20
		else:
			self.statuscounter = 50
		if self.armor <= 0:
			if sprites.count( self ) > 0:
				self.removeself( missiondata )
			return 1
			
	def removeself( self, missiondata ):
		sprites = missiondata.sprites
		try:
			sprites.remove( self )
		except:
			pass
		if self.status == FREEZE and self.spawnonfreeze != None:
			sprites.append( self.spawnonfreeze )
			self.spawnonfreeze.x = self.x
			self.spawnonfreeze.y = self.y
		else:
			if self.type != 'explosion':
				if self.lasthit == PLAYER:
					missiondata.playerdestroyed.append( self.type )
				else:
					missiondata.destroyed.append( self.type )
			if self.spawnondeath != None:
				if isinstance( self.spawnondeath, list ):
					for item in self.spawnondeath:
						sprites.append( item )
						selfdata = data.info[self.type]
						item.x = Random(int(self.x-selfdata.w/2),int(self.x+selfdata.w/2))
						item.y = Random(int(self.y-selfdata.h/2),int(self.y+selfdata.h/2))
				else:
					sprites.append( self.spawnondeath )
					self.spawnondeath.x = self.x
					self.spawnondeath.y = self.y



class Explosion(Particle):
	w, h = 0, 0
	size = MEDIUM
	side = BOTH
	armor = 1000000
	type = 'explosion'
	lifecounter = 0
	statusgiven = HIT
	obstructing = True
	projectile = False
	
	def __init__(self, drawsize=50, x=0.0, y=0.0, dmggiven=5, lifetime=50 ):
		self.drawsize = drawsize
		self.w = drawsize*2
		self.h = drawsize*2
		self.x = x*1.0
		self.y = y*1.0
		self.dmggiven = dmggiven
		self.lifetime = lifetime
	
	def blit( self ):
		self.w = self.drawsize
		self.h = self.drawsize
		gfx.drawExplosion( self )
	

		
class Object:
	acc = 0.1
	maxSpeed = 3.0
	w = 0
	h = 0
	statusgiven = HIT
	obstructing = True
	projectile = False
	freezable = True
	weapon = None
	
	def __init__( self, imgfile, side, armormax, dmggiven=10, size=MEDIUM, w=0, h=0, obstructing = True, maxSpeed=3.0, acc=0.1, statusgiven = HIT,freezable = True, projectile = False, weapon = None ):
		if isinstance( imgfile, "".__class__ ):
			self.img = Image( "./graphics/sprites/"+imgfile )
			self.w = self.img.width
			self.h = self.img.height
		else:
			self.blit = imgfile
			self.w = w
			self.h = h
		self.armormax = armormax
		self.dmggiven = dmggiven
		self.statusgiven = statusgiven
		self.maxSpeed = maxSpeed
		self.acc = acc
		self.obstructing = obstructing
		self.size = size
		self.side = side
		self.freezable = freezable
		self.projectile = projectile
		self.weapon = weapon
	
	def blit( self, obj ):
		if obj.status == HIT or obj.status == BURN:
			Video.TintBlit( self.img, int(obj.x-self.w/2), int(obj.y-self.h/2), RGB(255,150,150) )
		elif obj.status == FREEZE:
			Video.TintBlit( self.img, int(obj.x-self.w/2), int(obj.y-self.h/2), RGB(150,150,255) )
		else:
			self.img.Blit( int(obj.x-self.w/2), int(obj.y-self.h/2) )



LEFTMOST = 0
LEFT = 1
MIDDLE = 2
RIGHT = 3
RIGHTMOST = 4

# Armor and shield levels:
# 1: 1000
# 2: 1500
# 3: 2000
# 4: 2500
class Fighter(Object):
	armormax = 0
	armorclass = 1
	shieldclass = 1
	genlvl = 1
	dmggiven = 10
	side = PLAYER
	size = MEDIUM
	
	def __init__(self, imgfile):
		self.imgfile = imgfile
		tempimg = Canvas("./graphics/sprites/"+imgfile)
		self.w = tempimg.width
		self.h = tempimg.height/5
		self.weapons = ['','']
		self.gadgets = ['','']
		self.generator = 1
		self.img = []
		for i in range(5):
			self.img.append( Canvas(self.w,self.h) )
			tempimg.Clip(0,self.h * i, self.w, self.h)
			tempimg.Blit( self.img[i], 0, 0 )
			self.img[i] = Image( self.img[i] )
	
	def blit(self, obj):
		img = self.img
		if obj.status == HIT or obj.status == BURN:
			Video.TintBlit(self.img[obj.frame],int(obj.x-self.w/2), int(obj.y-self.h/2),RGB(255,150,150))
		else:
			img[obj.frame].Blit(int(obj.x-self.w/2),int(obj.y-self.h/2))
			if obj.status == SHIELD:
				Video.DrawEllipse(int(obj.x), int(obj.y),int(self.w*0.7),int(self.h*0.7),RGB(100,200,250,150),1)


class Item:
	delaycounter = 0
	def __init__(self, icon, name, description, energyuse, delay, price, autoactivated=False ):
		self.icon = Image("./graphics/icons/"+icon)
		self.name = name
		self.description = description
		self.energyuse = energyuse
		self.price = price
		self.delay = delay
		self.autoactivated = autoactivated
	
	def showinfo( self, font ):
		font.Print( 10,335,self.description+"\nEnergy use: "+`self.energyuse` )
		
	def checkactivate(self, button, missiondata, num):
		sprites = missiondata.sprites
		fpart = missiondata.fpart
		if self.delaycounter:
			self.delaycounter -= 1
		if 4 - fpart.genused.count(0) + self.energyuse <= data.info[fpart.type].genlvl:
			if button.Position() or self.autoactivated:
				temp = self.energyuse
				i = 0
				while temp:
					if fpart.genused[i] == 0:
						fpart.genused[i] = num
						temp -= 1
					i += 1
				if not self.delaycounter:
					self.delaycounter = self.delay
					weapons.use( self.name, missiondata )

class Weapon(Item):
	dmg = 0
	delay = 0
	delaycounter = 0
	
	
class Gadget(Item):
	pass
	

class Mission:
	def __init__( self, name, description, bg = 'StarField', limit = 0, missionneeded=None,weaponsneeded=None,gadgetsneeded=None,reputneeded=None,skillsneeded=None,special=False ):
		self.name = name
		self.description = description
		self.bg = bg
		self.limit = limit						# 0 means no limits on how many times you can take the mission!
		self.missionneeded = missionneeded		# what missions must you already have done, to do unlock this mission?
		self.weaponsneeded = weaponsneeded
		self.gagdetsneeded = gadgetsneeded
		self.reputneeded = reputneeded			# A tuple or list: [0] is lower reput, [1] is higher. Inclusive.
		self.skillsneeded = skillsneeded		# Just a single number
		self.special = special
		if weaponsneeded != None:
			self.description += "\nWeapons needed: "
			for item in weaponsneeded:
				self.description += data.info[ item ].name+"\t"
		if gadgetsneeded != None:
			self.description += "\nGadgets needed: "
			for item in gadgetsneeded:
				self.description += data.info[ item ].name+"\t"
	def showinfo( self ):
		plainfnt.Print( 10,335, self.description )

class StarField:
	def __init__( self, numstars=100 ):
		self.stars = []
		for i in range(numstars):
			self.stars.append( Coords( Random(0,Video.xres)*1.0, Random(0,Video.yres)*1.0 ) )
	def move( self, xmove, ymove ):
		for i in range(len(self.stars)):
			star = self.stars[i]
			moverate = (i%10) / 10.0 + 0.3
			star.x += xmove*moverate/2.0
			star.y += ymove*moverate/2.0
			if star.x < 0:
				star.x += Video.xres
			if star.x > Video.xres:
				star.x -= Video.xres
			if star.y < 0:
				star.y += Video.yres
			if star.y > Video.yres:
				star.y -= Video.yres
	def draw( self ):
		for star in self.stars:
			if Random(0,10):
				Video.DrawEllipse( int(star.x), int(star.y), 1,1, RGB(255,255,0), 1 )
			else:
				Video.DrawPixel( int(star.x), int(star.y), RGB(255,255,0) )


class SandField:
	def __init__( self ):
		self.x, self.y = 0.0, 0.0
		self.sandpic = Image("./graphics/bg/sand.png")
		
	def move( self, xmove, ymove ):
		self.x += xmove
		self.y += ymove
		self.x %= self.sandpic.width
		self.y %= self.sandpic.height
		
	def draw( self ):
		for x in range(int(self.x-self.sandpic.width),Video.xres,self.sandpic.width):
			for y in range(int(self.y-self.sandpic.height),Video.yres,self.sandpic.height):
				self.sandpic.Blit(int(x),int(y))
		
		
class Planet:
	def __init__( self, name, namepic, planetpic, missions = [], shopitems = [],upgrademax=1,travelto=[],price=10000 ):
		self.name = name
		self.namepic = Image( "./graphics/"+namepic )
		self.planetpic = Image( "./graphics/"+planetpic )
		self.upgrademax = upgrademax	# What is the maximum level you can upgrade your fighter to in this planet?
		self.missions = missions
		self.shopitems = shopitems
		self.price = price
		self.travelto = travelto
