from ika import *
import classes
import data
import flight
import weapons

LEFTMOST = 0
LEFT = 1
MIDDLE = 2
RIGHT = 3
RIGHTMOST = 4

def empty( particle,missiondata ):
	pass

def keyFighterMove(particle, missiondata):
	sprites = missiondata.sprites
	fighter = data.info['fighter']

	if Input.left.Position():
		particle.xSpeed -= fighter.acc
	elif particle.xSpeed < 0:
		particle.xSpeed += fighter.acc

	if Input.right.Position():
		particle.xSpeed += fighter.acc
	elif particle.xSpeed > 0:
		particle.xSpeed -= fighter.acc

	if Input.up.Position():
		particle.ySpeed -= fighter.acc
	elif particle.ySpeed < 0:
		particle.ySpeed += fighter.acc

	if Input.down.Position():
		particle.ySpeed += fighter.acc
	elif particle.ySpeed > 0:
		particle.ySpeed -= fighter.acc

	speedBlock(particle)
	fighterFrame(particle)
	stopSlow(particle)
	
	if particle.move():	#if out of Videobounds
		block(particle)
	
def keyFighterAttack(particle, missiondata):
	sprites = missiondata.sprites
	fighter = data.info['fighter']
	sprites[0].genused = [0,0,0,0]
	data.info[fighter.weapons[0]].checkactivate( Input['A'], missiondata, 1 )
	data.info[fighter.weapons[1]].checkactivate( Input['S'], missiondata, 2 )
	data.info[fighter.gadgets[0]].checkactivate( Input['D'], missiondata, 3 )
	data.info[fighter.gadgets[1]].checkactivate( Input['F'], missiondata, 4 )
	

def moveConst(particle, missiondata):
	sprites = missiondata.sprites
	speedBlock( particle )
	if particle.move() == 2:
		missiondata.notdestroyed.append( particle.type )
		try:
			sprites.remove(particle)
		except:
			pass


def moveAccelerate(particle, missiondata):
	sprites = missiondata.sprites
	speedBlock( particle )
	particle.xSpeed += particle.xAcc
	particle.ySpeed += particle.yAcc
	if particle.move() == 2:
		missiondata.notdestroyed.append( particle.type )
		try:
			sprites.remove(particle)
		except:
			pass

def moveBackForth(particle, missiondata):
	sprites = missiondata.sprites
	particleinfo = data.info[particle.type]
	if particle.dir == 0:
		particle.xSpeed -= particleinfo.acc
	if particle.dir == 1:
		particle.xSpeed += particleinfo.acc
	if particle.y < particleinfo.h/2:
		particle.ySpeed += particleinfo.acc
	elif particle.ySpeed > 0:
		particle.ySpeed -= particleinfo.acc
	if particle.y > Video.yres-particleinfo.h/2:
		particle.ySpeed -= particleinfo.acc
	elif particle.ySpeed < 0:
		particle.ySpeed += particleinfo.acc
	speedBlock(particle)
	particle.move()
	if particle.x < particleinfo.w/2:
		particle.dir = 1
	if particle.x > Video.xres - particleinfo.w/2:
		particle.dir = 0
	fighterFrame(particle)

def moveFacePlayer(particle, missiondata):
	sprites = missiondata.sprites
	particleinfo = data.info[particle.type]
	particle.xAcc = (missiondata.fpart.x-particle.x)/10.0
	particle.xSpeed += particle.xAcc
	if particle.y < particleinfo.h:
		particle.ySpeed += particleinfo.acc
	elif particle.ySpeed > 0:
		particle.ySpeed -= particleinfo.acc
	if particle.y > Video.yres-particleinfo.h or particle.y > missiondata.fpart.y-200:
		particle.ySpeed -= particleinfo.acc
	elif particle.ySpeed < 0:
		particle.ySpeed += particleinfo.acc
	speedBlock(particle)
	particle.move()

def moveBehindPlayer(particle, missiondata):
	sprites = missiondata.sprites
	particleinfo = data.info[particle.type]
	if particle.x < missiondata.fpart.x-30:
		particle.xSpeed += particleinfo.acc
	elif particle.xSpeed > 0:
		particle.xSpeed -= particleinfo.acc
	if particle.x > missiondata.fpart.x+30:
		particle.xSpeed -= particleinfo.acc
	elif particle.xSpeed < 0:
		particle.xSpeed += particleinfo.acc
	if particle.y < particleinfo.h or particle.y < missiondata.fpart.y+100:
		particle.ySpeed += particleinfo.acc
	elif particle.ySpeed > 0:
		particle.ySpeed -= particleinfo.acc
	if particle.y > Video.yres-particleinfo.h or particle.y > missiondata.fpart.y+200:
		particle.ySpeed -= particleinfo.acc
	elif particle.ySpeed < 0:
		particle.ySpeed += particleinfo.acc
	speedBlock( particle )
	stopSlow( particle )
	particle.move()
	
def moveWave(particle, missiondata):
	sprites = missiondata.sprites
	particleinfo = data.info[particle.type]
	if particle.y < 150:
		particle.ySpeed += particleinfo.acc/2
	if particle.y > 150:
		particle.ySpeed -= particleinfo.acc/2
	if particle.dir == 0:
		particle.xSpeed -= particleinfo.acc
	if particle.dir == 1:
		particle.xSpeed += particleinfo.acc
	speedBlock(particle)
	particle.move()
	if particle.x < particleinfo.w/2:
		particle.dir = 1
	if particle.x > Video.xres - particleinfo.w/2:
		particle.dir = 0
	fighterFrame(particle)

def followCreated(particle, missiondata):
	sprites = missiondata.sprites
	particleinfo = data.info[particle.type]
	head = particle.created
	if head.armor > 0:
		headinfo = data.info[head.type]
		particle.x = (head.x-particle.x)/10+particle.x
		particle.y = (head.y-particle.y)/10+particle.y
	else:
		particle.removeself( missiondata )


def attackConst( particle, missiondata ):
	exec( "weapons.en"+data.info[particle.type].weapon+"(particle, missiondata)" )


# **************************
# General checking functions
# **************************
def speedBlock(particle):
	type = data.info[particle.type]
	# Check that the particle.isn't moving faster than allowed
	if particle.xSpeed > type.maxSpeed:
		particle.xSpeed = type.maxSpeed
	if particle.xSpeed < -type.maxSpeed:
		particle.xSpeed = -type.maxSpeed
	if particle.ySpeed > type.maxSpeed:
		particle.ySpeed = type.maxSpeed
	if particle.ySpeed < -type.maxSpeed:
		particle.ySpeed = -type.maxSpeed


def fighterFrame(particle):
	type = data.info[particle.type]
	if particle.xSpeed > type.maxSpeed/5:
		particle.frame = RIGHT
	elif particle.xSpeed < -type.maxSpeed/5:
		particle.frame = LEFT
	else:
		particle.frame = MIDDLE
	if particle.xSpeed >= type.maxSpeed:
		particle.frame = RIGHTMOST
	if particle.xSpeed <= -type.maxSpeed:
		particle.frame = LEFTMOST



def stopSlow(particle):
	type = data.info[particle.type]
	# Stop the ship if the speed is very low
	if abs(particle.xSpeed) < type.acc:
		particle.xSpeed = 0.0
	if abs(particle.ySpeed) < type.acc:
		particle.ySpeed = 0.0



SLACK = 5
def block(particle):
	type = data.info[particle.type]
	if particle.x < type.w/2 - SLACK:
		particle.x = type.w/2 - SLACK
		if particle.xSpeed < 0:
			particle.xSpeed = 0
	
	if particle.x > Video.xres - type.w/2 + SLACK:
		particle.x = Video.xres - type.w/2 + SLACK
		if particle.xSpeed > 0:
			particle.xSpeed = 0
			
	if particle.y < type.h/2-SLACK:
		particle.y = type.h/2-SLACK
		if particle.ySpeed < 0:
			particle.ySpeed = 0
	
	if particle.y > Video.yres - type.h/2 + SLACK:
		particle.y = Video.yres - type.h/2 + SLACK
		if particle.ySpeed > 0:
			particle.ySpeed = 0
